# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from ..types import PathParamMultipleResponse, PathParamSingularResponse
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._resource import SyncAPIResource, AsyncAPIResource
from .._base_client import make_request_options

__all__ = ["PathParams", "AsyncPathParams"]


class PathParams(SyncAPIResource):
    def multiple(
        self,
        last: str,
        *,
        first: str,
        second: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PathParamMultipleResponse:
        """Endpoint with multiple path parameters."""
        return self._post(
            f"/path_params/{first}/{second}/{last}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PathParamMultipleResponse,
        )

    def singular(
        self,
        singular: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PathParamSingularResponse:
        """Endpoint with a singular path parameter."""
        return self._post(
            f"/path_params/{singular}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PathParamSingularResponse,
        )


class AsyncPathParams(AsyncAPIResource):
    async def multiple(
        self,
        last: str,
        *,
        first: str,
        second: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PathParamMultipleResponse:
        """Endpoint with multiple path parameters."""
        return await self._post(
            f"/path_params/{first}/{second}/{last}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PathParamMultipleResponse,
        )

    async def singular(
        self,
        singular: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | None | NotGiven = NOT_GIVEN,
        idempotency_key: str | None = None,
    ) -> PathParamSingularResponse:
        """Endpoint with a singular path parameter."""
        return await self._post(
            f"/path_params/{singular}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=PathParamSingularResponse,
        )
